package com.akdeniz.googleplaycrawler;

import java.io.IOException;

public class GooglePlayException extends IOException {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public GooglePlayException(String message) {
		super(message);
	}
}
